#!/usr/bin/env python2

import popen2
import string
import re
import os
import pwd
import sys


# FIXME: Remove this function when Python provides realpath. 
def realpath(filename):
    """Return the canonical path of the specified filename, eliminating any
symbolic links encountered in the path."""
    filename = os.path.abspath(filename)

    bits = ['/'] + string.split(filename, '/')[1:]
    for i in range(2, len(bits)+1):
        component = apply(os.path.join, bits[0:i])
        if os.path.islink(component):
            resolved = os.readlink(component)
            (dir, file) = os.path.split(component)
            resolved = os.path.normpath(os.path.join(dir, resolved))
            newpath = apply(os.path.join, [resolved] + bits[i:])
            return realpath(newpath)

    return filename


def get_origin_dir(argv0=sys.argv[0]):
    """Get program origin directory"""
    
    abs_path = realpath(os.path.abspath(argv0))

    return os.path.dirname(abs_path)


def get_httpd_config():
    p = popen2.Popen3("httpd -V", 1)
    while 1:
        line = p.fromchild.readline()
        if not line:
            break

        match = re.search("SERVER_CONFIG_FILE=\"(.*)\"", line)
        if match:
            httpd_config = match.group(1)

    return httpd_config


def get_httpd_doc_root(config_file):
    f = open(config_file, "r")
    while 1:
        line = f.readline()
        if not line:
            break

        match = re.search("^DocumentRoot\s+(.*)", line)
        if match:
            doc_root = match.group(1)

    if doc_root[0] == '"' and doc_root[-1] == '"':
        doc_root = doc_root[1:-1]
    
    return doc_root

def get_apache_user(config_file):
    f = open(config_file, "r")
    while 1:
        line = f.readline()
        if not line:
            break

        match = re.search("^User\s+(.*)", line)
        if match:
            user = match.group(1)

    return user


def lexists(path):
    try:
        os.lstat(path)
    except os.error:
        return 0
    return 1


def main():
    print "Finding WebKOM directory...",
    webkom_dir = get_origin_dir()
    print webkom_dir
    
    print "Finding httpd.conf...",
    httpd_conf = get_httpd_config()
    print httpd_conf

    print "Finding DocumentRoot...",
    doc_root = get_httpd_doc_root(httpd_conf)
    print doc_root

    web_home = os.path.join(doc_root, "webkom")
    print "Creating directory", web_home
    if not lexists(web_home):
        os.mkdir(web_home, 0755)
    else:
        print web_home, "exists, skipping."

    src_img_dir = os.path.join(webkom_dir, "images")
    dst_img_dir = os.path.join(web_home, "images")
    print "Creating link", dst_img_dir, "->", src_img_dir
    if lexists(dst_img_dir):
        print dst_img_dir, "exists, overwriting."
        os.remove(dst_img_dir)
    os.symlink(src_img_dir, dst_img_dir)

    wrapper_file = os.path.join(web_home, "webkom.py.wrapper")
    print "Creating", wrapper_file, "(change Python-interpreter in this file)"
    f = open(wrapper_file, "w")
    f.write("#!/bin/sh\n")
    f.write("# This is the place where to select Python interpreter for WebKOM.\n")
    f.write("PATH=$PATH\n")
    f.write("export PATH\n")
    f.write("PYTHONOPTIMIZE=1\n")
    f.write("export PYTHONOPTIMIZE\n")
    f.write("exec " + os.path.join(webkom_dir, "webkom.py") + "\n")
    f.close()
    os.chmod(wrapper_file, 0775)
    

    lnk_src = "webkom.py.wrapper"
    lnk_dst = os.path.join(web_home, "webkom.py")
    print "Creating link", lnk_dst, "->", lnk_src
    if lexists(lnk_dst):
        print lnk_dst, "exists, overwriting."
        os.remove(lnk_dst)
    os.symlink(lnk_src, lnk_dst)

    print "Finding Apache user...", 
    apache_user = get_apache_user(httpd_conf)
    print apache_user

    log_dir = os.path.join(webkom_dir, "logs")
    print "Changing owner of", log_dir, "to", apache_user
    cur_gid = os.stat(log_dir)[5]
    new_uid = pwd.getpwnam(apache_user)[2]
    os.chown(log_dir, new_uid, cur_gid)

    print "Add the lines below to", httpd_conf, ":"
    conf_clip = """
FastCgiServer %s
<Directory %s>
Options FollowSymLinks
DirectoryIndex webkom.py
Allow from all
<Files webkom.py>
    SetHandler fastcgi-script
</Files>
</Directory>""" % (os.path.join(web_home, "webkom.py"), # /home/httpd/html/webkom/webkom.py
                   web_home) # /home/httpd/html/webkom

    print conf_clip

    
if __name__=="__main__":
    main()
    


